==> xknx-3.0.0-py3-none-any.whl: xknx/core/group_address_dpt.py <==
"""Group address data point type table."""

from __future__ import annotations

from collections.abc import Mapping
import logging

from xknx.dpt.dpt import DPTBase, _DPTMainSubDict
from xknx.exceptions import ConversionError, CouldNotParseAddress, CouldNotParseTelegram
from xknx.telegram import Telegram, TelegramDecodedData
from xknx.telegram.address import (
    DeviceAddressableType,
    DeviceGroupAddress,
    GroupAddress,
    InternalGroupAddress,
    parse_device_group_address,
)
from xknx.telegram.apci import GroupValueResponse, GroupValueWrite

_GA_DPT_LOGGER = logging.getLogger("xknx.ga_dpt")


class GroupAddressDPT:
    """Class for mapping group addresses to data point types for eager decoding."""

    __slots__ = ("_ga_dpts",)

    def __init__(self) -> None:
        """Initialize GADataTypes class."""
        # using dict[int | str] instead of dict[DeviceGroupAddress] is faster.
        self._ga_dpts: dict[int | str, type[DPTBase]] = {}

    def set(
        self,
        ga_dpt: Mapping[DeviceAddressableType, int | str | _DPTMainSubDict],
    ) -> None:
        """Assign decoders to group addresses."""
        unknown_dpts = set()
        for addr, dpt in ga_dpt.items():
            try:
                address = parse_device_group_address(addr)
            except CouldNotParseAddress as err:
                _GA_DPT_LOGGER.warning("Invalid group address %s: %s", addr, err)
                continue
            if (transcoder := DPTBase.parse_transcoder(dpt)) is None:
                unknown_dpts.add(repr(dpt))  # prevent unhashable types (dict)
                continue
            self._ga_dpts[address.raw] = transcoder
        if unknown_dpts:
            _GA_DPT_LOGGER.debug("No transcoder found for DPTs: %s", unknown_dpts)

    def get(self, address: DeviceGroupAddress) -> type[DPTBase] | None:
        """Return transcoder for group address."""
        return self._ga_dpts.get(address.raw)

    def clear(self) -> None:
        """Clear all group addresses."""
        self._ga_dpts = {}

    def set_decoded_data(self, telegram: Telegram) -> None:
        """Update telegram data with decoded value."""
        if telegram.decoded_data is not None:
            return
        if not isinstance(telegram.payload, GroupValueWrite | GroupValueResponse):
            return
        assert isinstance(  # GroupValueWrite and GroupValueResponse can not have IndividualAddress
            telegram.destination_address, GroupAddress | InternalGroupAddress
        )
        if (transcoder := self.get(telegram.destination_address)) is None:
            return
        try:
            value = transcoder.from_knx(telegram.payload.value)
        except (CouldNotParseTelegram, ConversionError) as err:
            _GA_DPT_LOGGER.warning(
                "DPT decoding error for %s of %s: %s", transcoder, self, err
            )
            return
        telegram.decoded_data = TelegramDecodedData(transcoder, value)

==> xknx-3.1.1-py3-none-any.whl: xknx/core/group_address_dpt.py <==
"""Group address data point type table."""

from __future__ import annotations

from collections.abc import Mapping
import logging

from xknx.dpt.dpt import DPTBase, _DPTMainSubDict
from xknx.exceptions import ConversionError, CouldNotParseAddress, CouldNotParseTelegram
from xknx.telegram import Telegram, TelegramDecodedData
from xknx.telegram.address import (
    DeviceAddressableType,
    DeviceGroupAddress,
    GroupAddress,
    InternalGroupAddress,
    parse_device_group_address,
)
from xknx.telegram.apci import GroupValueResponse, GroupValueWrite

_GA_DPT_LOGGER = logging.getLogger("xknx.ga_dpt")


class GroupAddressDPT:
    """Class for mapping group addresses to data point types for eager decoding."""

    __slots__ = ("_ga_dpts",)

    def __init__(self) -> None:
        """Initialize GADataTypes class."""
        # using dict[int | str] instead of dict[DeviceGroupAddress] is faster.
        self._ga_dpts: dict[int | str, type[DPTBase]] = {}

    def set(
        self,
        ga_dpt: Mapping[DeviceAddressableType, int | str | _DPTMainSubDict],
    ) -> None:
        """Assign decoders to group addresses."""
        unknown_dpts = set()
        for addr, dpt in ga_dpt.items():
            try:
                address = parse_device_group_address(addr)
            except CouldNotParseAddress as err:
                _GA_DPT_LOGGER.warning("Invalid group address %s: %s", addr, err)
                continue
            if (transcoder := DPTBase.parse_transcoder(dpt)) is None:
                unknown_dpts.add(repr(dpt))  # prevent unhashable types (dict)
                continue
            self._ga_dpts[address.raw] = transcoder
        if unknown_dpts:
            _GA_DPT_LOGGER.debug("No transcoder found for DPTs: %s", unknown_dpts)

    def get(self, address: DeviceGroupAddress) -> type[DPTBase] | None:
        """Return transcoder for group address."""
        return self._ga_dpts.get(address.raw)

    def clear(self) -> None:
        """Clear all group addresses."""
        self._ga_dpts = {}

    def set_decoded_data(self, telegram: Telegram) -> None:
        """Update telegram data with decoded value."""
        if telegram.decoded_data is not None:
            return
        if not isinstance(telegram.payload, GroupValueWrite | GroupValueResponse):
            return
        assert isinstance(  # GroupValueWrite and GroupValueResponse can not have IndividualAddress
            telegram.destination_address, GroupAddress | InternalGroupAddress
        )
        if (transcoder := self.get(telegram.destination_address)) is None:
            return
        try:
            value = transcoder.from_knx(telegram.payload.value)
        except (CouldNotParseTelegram, ConversionError) as err:
            _GA_DPT_LOGGER.warning(
                "DPT decoding error. Telegram from %s to %s with payload %s can't be decoded by %s: %s",
                telegram.source_address,
                telegram.destination_address,
                telegram.payload.value,
                transcoder.__name__,
                err,
            )
            return
        telegram.decoded_data = TelegramDecodedData(transcoder, value)
